@@ -3,19 +3,18 @@
 use rustc_data_structures::fx::FxHashMap as HashMap;
 use rustc_index::{
   bit_set::{HybridBitSet, HybridIter, SparseBitMatrix},
-  vec::{Enumerated, Idx, IndexVec},
+  vec::{Idx, IndexVec},
 };
 
 use rustc_mir_dataflow::{fmt::DebugWithContext, JoinSemiLattice};
 use std::{
-  fmt,
+  fmt::{self},
   hash::Hash,
   ops::{Deref, DerefMut},
   rc::Rc,
-  slice::Iter,
 };
 
-pub trait IndexedValue: Eq + Hash + Clone + fmt::Debug {
+pub trait IndexedValue: Eq + Hash + Clone + Ord + fmt::Debug {
   type Index: Idx;
   type Domain: IndexedDomain<Index = Self::Index, Value = Self> = DefaultDomain<Self::Index, Self>;
 }
@@ -30,6 +29,12 @@ impl<T: IndexedValue> ToIndex<T> for T {
   }
 }
 
+impl<T: IndexedValue> ToIndex<T> for &T {
+  fn to_index(&self, domain: &T::Domain) -> T::Index {
+    domain.index(self)
+  }
+}
+
 // Can't make this a blanket impl b/c it conflicts with the blanket impl above :(
 #[macro_export]
 macro_rules! to_index_impl {
@@ -47,8 +52,8 @@ pub trait IndexedDomain {
   type Index: Idx = <Self::Value as IndexedValue>::Index;
   fn value(&self, index: Self::Index) -> &Self::Value;
   fn index(&self, value: &Self::Value) -> Self::Index;
-  fn len(&self) -> usize;
-  fn iter_enumerated<'a>(&'a self) -> Enumerated<Self::Index, Iter<'a, Self::Value>>;
+  fn contains(&self, value: &Self::Value) -> bool;
+  fn as_vec(&self) -> &IndexVec<Self::Index, Self::Value>;
 }
 
 #[derive(Clone)]
@@ -85,13 +90,12 @@ impl<I: Idx, T: IndexedValue> IndexedDomain for DefaultDomain<I, T> {
       .get(value)
       .unwrap_or_else(|| panic!("No index for value: {:?}", value))
   }
-
-  fn len(&self) -> usize {
-    self.index_to_value.len()
+  fn contains(&self, value: &T) -> bool {
+    self.value_to_index.contains_key(value)
   }
 
-  fn iter_enumerated<'a>(&'a self) -> Enumerated<Self::Index, Iter<'a, Self::Value>> {
-    self.index_to_value.iter_enumerated()
+  fn as_vec(&self) -> &IndexVec<Self::Index, Self::Value> {
+    &self.index_to_value
   }
 }
 
@@ -137,7 +141,7 @@ pub struct IndexSet<T: IndexedValue, S = OwnedSet<T>> {
 impl<T: IndexedValue> IndexSet<T, OwnedSet<T>> {
   pub fn new(domain: Rc<T::Domain>) -> Self {
     IndexSet {
-      set: OwnedSet(HybridBitSet::new_empty(domain.len())),
+      set: OwnedSet(HybridBitSet::new_empty(domain.as_vec().len())),
       domain,
     }
   }
@@ -259,11 +263,24 @@ impl<T: IndexedValue + fmt::Debug, S: ToSet<T>> fmt::Debug for IndexSet<T, S> {
   }
 }
 
+struct Escape<T>(T);
+impl<T: fmt::Debug> fmt::Display for Escape<T> {
+  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+    write!(f, "{}", html_escape::encode_text(&format!("{:?}", self.0)))
+  }
+}
+
 impl<T: IndexedValue + fmt::Debug, S: ToSet<T>, C> DebugWithContext<C> for IndexSet<T, S>
 where
   T::Index: ToIndex<T>,
 {
-  fn fmt_diff_with(&self, old: &Self, _ctxt: &C, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+  fn fmt_with(&self, _ctxt: &C, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+    let mut elts = self.iter().collect::<Vec<_>>();
+    elts.sort();
+    write!(f, "{}", Escape(elts))
+  }
+
+  fn fmt_diff_with(&self, old: &Self, ctxt: &C, f: &mut fmt::Formatter<'_>) -> fmt::Result {
     if self == old {
       return Ok(());
     }
@@ -278,11 +295,13 @@ where
       .collect_indices(self.domain.clone());
 
     if added.len() > 0 {
-      write!(f, "\u{001f}+{:?}", added)?;
+      write!(f, "\u{001f}+")?;
+      added.fmt_with(ctxt, f)?;
     }
 
     if removed.len() > 0 {
-      write!(f, "\u{001f}-{:?}", removed)?;
+      write!(f, "\u{001f}-")?;
+      removed.fmt_with(ctxt, f)?;
     }
 
     Ok(())
@@ -311,13 +330,13 @@ where
 pub struct IndexMatrix<R: IndexedValue, C: IndexedValue> {
   matrix: SparseBitMatrix<R::Index, C::Index>,
   row_domain: Rc<R::Domain>,
-  col_domain: Rc<C::Domain>,
+  pub col_domain: Rc<C::Domain>,
 }
 
 impl<R: IndexedValue, C: IndexedValue> IndexMatrix<R, C> {
   pub fn new(row_domain: Rc<R::Domain>, col_domain: Rc<C::Domain>) -> Self {
     IndexMatrix {
-      matrix: SparseBitMatrix::new(col_domain.len()),
+      matrix: SparseBitMatrix::new(col_domain.as_vec().len()),
       row_domain,
       col_domain,
     }
@@ -398,7 +417,10 @@ impl<R: IndexedValue, C: IndexedValue> JoinSemiLattice for IndexMatrix<R, C> {
   }
 }
 
-impl<R: IndexedValue + fmt::Debug, C: IndexedValue + fmt::Debug> fmt::Debug for IndexMatrix<R, C> {
+impl<R: IndexedValue + fmt::Debug, C: IndexedValue + fmt::Debug> fmt::Debug for IndexMatrix<R, C>
+where
+  R::Index: ToIndex<R>,
+{
   fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
     write!(f, "{{")?;
 
@@ -408,17 +430,15 @@ impl<R: IndexedValue + fmt::Debug, C: IndexedValue + fmt::Debug> fmt::Debug for
         continue;
       }
 
-      write!(f, "  {:?}: [", self.row_domain.value(row))?;
-      for (i, col) in self.matrix.iter(row).enumerate() {
-        write!(f, "{:?}", self.col_domain.value(col))?;
-        if i < n - 1 {
-          write!(f, ", ")?;
-        }
-      }
-      write!(f, "]<br align=\"left\" />")?;
+      write!(
+        f,
+        "  {:?}: {:?},",
+        self.row_domain.value(row),
+        self.row_set(row).unwrap()
+      )?;
     }
 
-    write!(f, "}}<br align=\"left\" />")
+    write!(f, "}}")
   }
 }
 
@@ -428,6 +448,24 @@ where
   R::Index: ToIndex<R>,
   C::Index: ToIndex<C>,
 {
+  fn fmt_with(&self, ctxt: &Ctx, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+    write!(f, "{{")?;
+
+    for row in self.matrix.rows() {
+      if let Some(row_set) = self.row_set(row) {
+        if row_set.len() == 0 {
+          continue;
+        }
+
+        write!(f, "  {}: ", Escape(self.row_domain.value(row)))?;
+        row_set.fmt_with(ctxt, f)?;
+        write!(f, "]<br align=\"left\" />")?;
+      }
+    }
+
+    write!(f, "}}<br align=\"left\" />")
+  }
+
   fn fmt_diff_with(&self, old: &Self, ctxt: &Ctx, f: &mut fmt::Formatter<'_>) -> fmt::Result {
     if self == old {
       return Ok(());
@@ -447,7 +485,7 @@ where
         continue;
       }
 
-      write!(f, "{:?}: ", row_value)?;
+      write!(f, "{}: ", Escape(row_value))?;
       set.fmt_diff_with(old_set, ctxt, f)?;
       writeln!(f)?;
     }
