@@ -1,29 +1,28 @@
-#![allow(dead_code)]
-
 use anyhow::{bail, Result};
 use log::{trace, warn};
-use rustc_borrowck::consumers::BodyWithBorrowckFacts;
 use rustc_data_structures::fx::{FxHashMap as HashMap, FxHashSet as HashSet};
 use rustc_graphviz as dot;
-use rustc_hir::{def_id::DefId, BodyId};
+use rustc_hir::def_id::DefId;
 use rustc_middle::{
   mir::{
     pretty::write_mir_fn,
     visit::{MutatingUseContext, NonMutatingUseContext, NonUseContext, PlaceContext, Visitor},
     *,
   },
-  ty::{
-    self, RegionKind, RegionVid, Ty, TyCtxt, TyKind, TyS, TypeFoldable, TypeVisitor,
-    WithOptConstParam,
-  },
+  ty::{self, RegionKind, RegionVid, Ty, TyCtxt, TyKind, TyS, TypeFoldable, TypeVisitor},
 };
 use rustc_mir_dataflow::{fmt::DebugWithContext, graphviz, Analysis, Results};
 use rustc_mir_transform::MirPass;
 use rustc_span::Span;
 use rustc_target::abi::VariantIdx;
+use smallvec::SmallVec;
 use std::{
-  collections::hash_map::Entry, fs::File, hash::Hash, io::Write, ops::ControlFlow, process::Command,
+  collections::hash_map::Entry, hash::Hash, io::Write, ops::ControlFlow, path::Path,
+  process::Command,
 };
+use tempfile::NamedTempFile;
+
+use crate::core::extensions::{is_extension_active, MutabilityMode};
 
 pub fn operand_to_place(operand: &Operand<'tcx>) -> Option<Place<'tcx>> {
   match operand {
@@ -33,10 +32,7 @@ pub fn operand_to_place(operand: &Operand<'tcx>) -> Option<Place<'tcx>> {
 }
 
 pub fn local_to_place(local: Local, tcx: TyCtxt<'tcx>) -> Place<'tcx> {
-  Place {
-    local,
-    projection: tcx.intern_place_elems(&[]),
-  }
+  mk_place(local, &[], tcx)
 }
 
 struct CollectRegions<'tcx> {
@@ -48,6 +44,7 @@ struct CollectRegions<'tcx> {
   places: Option<HashSet<Place<'tcx>>>,
   types: Option<HashSet<Ty<'tcx>>>,
   regions: HashMap<RegionVid, Vec<(Place<'tcx>, Mutability)>>,
+  depth_limit: Option<usize>,
 }
 
 impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
@@ -64,7 +61,17 @@ impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
       return ControlFlow::Continue(());
     }
 
-    trace!("exploring local {:?} with {:?}", self.local, ty);
+    if let Some(limit) = self.depth_limit {
+      if self.place_stack.len() > limit {
+        return ControlFlow::Continue(());
+      }
+    }
+
+    trace!(
+      "exploring {:?} with {:?}",
+      mk_place(self.local, &self.place_stack, self.tcx),
+      ty
+    );
 
     self.ty_stack.push(ty);
 
@@ -161,10 +168,7 @@ impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
     };
 
     if let Some(places) = self.places.as_mut() {
-      places.insert(Place {
-        local: self.local,
-        projection: self.tcx.intern_place_elems(&self.place_stack),
-      });
+      places.insert(mk_place(self.local, &self.place_stack, self.tcx));
     }
 
     if let Some(types) = self.types.as_mut() {
@@ -189,10 +193,7 @@ impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
           Mutability::Mut
         };
 
-        let place = Place {
-          local: self.local,
-          projection: self.tcx.intern_place_elems(&self.place_stack),
-        };
+        let place = mk_place(self.local, &self.place_stack, self.tcx);
 
         self
           .regions
@@ -229,6 +230,7 @@ pub fn interior_pointers<'tcx>(
     regions: HashMap::default(),
     places: None,
     types: None,
+    depth_limit: None,
   };
   region_collector.visit_ty(ty);
   region_collector.regions
@@ -239,6 +241,7 @@ pub fn interior_places<'tcx>(
   tcx: TyCtxt<'tcx>,
   body: &Body<'tcx>,
   def_id: DefId,
+  depth_limit: Option<usize>,
 ) -> Vec<Place<'tcx>> {
   let ty = place.ty(body.local_decls(), tcx).ty;
   let mut region_collector = CollectRegions {
@@ -250,6 +253,7 @@ pub fn interior_places<'tcx>(
     regions: HashMap::default(),
     places: Some(HashSet::default()),
     types: None,
+    depth_limit,
   };
   region_collector.visit_ty(ty);
   region_collector.places.unwrap().into_iter().collect()
@@ -271,20 +275,22 @@ pub fn interior_types<'tcx>(
     regions: HashMap::default(),
     places: None,
     types: Some(HashSet::default()),
+    depth_limit: None,
   };
   region_collector.visit_ty(ty);
   region_collector.types.unwrap().into_iter().collect()
 }
 
 pub fn arg_mut_ptrs<'tcx>(
-  args: &[Place<'tcx>],
+  args: &[(usize, Place<'tcx>)],
   tcx: TyCtxt<'tcx>,
   body: &Body<'tcx>,
   def_id: DefId,
-) -> Vec<Place<'tcx>> {
+) -> Vec<(usize, Place<'tcx>)> {
+  let ignore_mut = is_extension_active(|mode| mode.mutability_mode == MutabilityMode::IgnoreMut);
   args
     .iter()
-    .map(|place| {
+    .map(|(i, place)| {
       interior_pointers(*place, tcx, body, def_id)
         .into_iter()
         .map(|(_, places)| {
@@ -292,20 +298,21 @@ pub fn arg_mut_ptrs<'tcx>(
             .into_iter()
             .filter_map(|(place, mutability)| match mutability {
               Mutability::Mut => Some(place),
-              Mutability::Not => None,
+              Mutability::Not => ignore_mut.then(|| place),
             })
         })
         .flatten()
-        .map(|place| tcx.mk_place_deref(place))
+        .map(move |place| (*i, tcx.mk_place_deref(place)))
     })
     .flatten()
     .collect::<Vec<_>>()
 }
 
-pub fn arg_places<'tcx>(args: &[Operand<'tcx>]) -> Vec<Place<'tcx>> {
+pub fn arg_places<'tcx>(args: &[Operand<'tcx>]) -> Vec<(usize, Place<'tcx>)> {
   args
     .iter()
-    .filter_map(|arg| operand_to_place(arg))
+    .enumerate()
+    .filter_map(|(i, arg)| operand_to_place(arg).map(move |place| (i, place)))
     .collect::<Vec<_>>()
 }
 
@@ -405,10 +412,7 @@ pub fn pointer_for_place(place: Place<'tcx>, tcx: TyCtxt<'tcx>) -> Option<Place<
     .iter_projections()
     .rev()
     .find(|(_, elem)| matches!(elem, ProjectionElem::Deref))
-    .map(|(place_ref, _)| Place {
-      local: place_ref.local,
-      projection: tcx.intern_place_elems(place_ref.projection),
-    })
+    .map(|(place_ref, _)| mk_place(place_ref.local, place_ref.projection, tcx))
 }
 
 pub fn span_to_places(body: &Body<'tcx>, span: Span) -> (Vec<(Place<'tcx>, Location)>, Vec<Span>) {
@@ -421,13 +425,6 @@ pub fn span_to_places(body: &Body<'tcx>, span: Span) -> (Vec<(Place<'tcx>, Locat
 
   impl Visitor<'tcx> for FindSpannedPlaces<'_, 'tcx> {
     fn visit_place(&mut self, place: &Place<'tcx>, context: PlaceContext, location: Location) {
-      // Assignments to return value have a span of the entire function, which we want to avoid
-      if let Some(local) = place.as_local() {
-        if local.as_usize() == 0 {
-          return;
-        }
-      }
-
       // Three cases, shown by example:
       //   fn foo(x: i32) {
       //     let y = x + 1;
@@ -442,22 +439,22 @@ pub fn span_to_places(body: &Body<'tcx>, span: Span) -> (Vec<(Place<'tcx>, Locat
         PlaceContext::MutatingUse(MutatingUseContext::Store)
         | PlaceContext::NonMutatingUse(NonMutatingUseContext::Inspect) => {
           let source_info = self.body.source_info(location);
-          Some(source_info.span)
+          source_info.span
         }
         PlaceContext::NonUse(NonUseContext::VarDebugInfo)
           if self.body.args_iter().any(|local| local == place.local) =>
         {
           let source_info = self.body.local_decls()[place.local].source_info;
-          Some(source_info.span)
+          source_info.span
+        }
+        _ => {
+          return;
         }
-        _ => None,
       };
 
-      if let Some(span) = span {
-        if self.span.contains(span) || span.contains(self.span) {
-          self.places.insert((*place, location));
-          self.place_spans.push(span);
-        }
+      if self.span.contains(span) || span.contains(self.span) {
+        self.places.insert((*place, location));
+        self.place_spans.push(span);
       }
     }
   }
@@ -489,9 +486,10 @@ pub fn span_to_places(body: &Body<'tcx>, span: Span) -> (Vec<(Place<'tcx>, Locat
 }
 
 pub fn dump_results<'tcx, A>(
-  path: &str,
   body: &Body<'tcx>,
   results: &Results<'tcx, A>,
+  _def_id: DefId,
+  _tcx: TyCtxt<'tcx>,
 ) -> Result<()>
 where
   A: Analysis<'tcx>,
@@ -500,14 +498,27 @@ where
   let graphviz = graphviz::Formatter::new(body, results, graphviz::OutputStyle::AfterOnly);
   let mut buf = Vec::new();
   dot::render(&graphviz, &mut buf)?;
-  let mut file = File::create("/tmp/graph.dot")?;
-  file.write_all(&buf)?;
+
+  let mut file = NamedTempFile::new()?;
+  file.as_file_mut().write_all(&buf)?;
+
+  let output_dir = Path::new("target");
+  // let fname = tcx.def_path_debug_str(def_id);
+  let fname = "results";
+  let output_path = output_dir.join(format!("{}.png", fname));
+
   let status = Command::new("dot")
-    .args(&["-Tpng", "/tmp/graph.dot", "-o", path])
+    .args(&[
+      "-Tpng",
+      &file.path().display().to_string(),
+      "-o",
+      &output_path.display().to_string(),
+    ])
     .status()?;
   if !status.success() {
-    bail!("dot for {} failed", path)
+    bail!("dot for {} failed", output_path.display())
   };
+
   Ok(())
 }
 
@@ -526,11 +537,11 @@ pub fn location_to_string(location: Location, body: &Body<'_>) -> String {
   }
 }
 
-struct SimplifyMir;
+pub struct SimplifyMir;
 impl MirPass<'tcx> for SimplifyMir {
   fn run_pass(&self, _tcx: TyCtxt<'tcx>, body: &mut Body<'tcx>) {
     for block in body.basic_blocks_mut() {
-      block.retain_statements(|stmt| {
+      block.statements.retain(|stmt| {
         !matches!(
           stmt.kind,
           // TODO: variable_select_lhs test fails if we remove FakeRead
@@ -553,40 +564,28 @@ impl MirPass<'tcx> for SimplifyMir {
   }
 }
 
-pub fn get_body_with_borrowck_facts(
-  tcx: TyCtxt<'tcx>,
-  body_id: BodyId,
-) -> BodyWithBorrowckFacts<'tcx> {
-  let local_def_id = tcx.hir().body_owner_def_id(body_id);
-  let mut body_with_facts = rustc_borrowck::consumers::get_body_with_borrowck_facts(
-    tcx,
-    WithOptConstParam::unknown(local_def_id),
-  );
-
-  let body = &mut body_with_facts.body;
-  SimplifyMir.run_pass(tcx, body);
-  // simplify::SimplifyCfg::new("flowistry").run_pass(tcx, body);
-
-  body_with_facts
+pub fn mk_place(local: Local, projection: &[PlaceElem<'tcx>], tcx: TyCtxt<'tcx>) -> Place<'tcx> {
+  Place {
+    local,
+    projection: tcx.intern_place_elems(projection),
+  }
 }
 
-pub fn split_deref(
-  place: Place<'tcx>,
-  tcx: TyCtxt<'tcx>,
-) -> Option<(Place<'tcx>, &'tcx [PlaceElem<'tcx>])> {
-  let deref_index = place
-    .projection
-    .iter()
-    .enumerate()
-    .rev()
-    .find(|(_, elem)| matches!(elem, ProjectionElem::Deref))
-    .map(|(i, _)| i)?;
+pub fn is_arg(place: Place<'tcx>, body: &Body<'tcx>) -> bool {
+  let i = place.local.as_usize();
+  i > 0 && i - 1 < body.arg_count
+}
 
-  Some((
-    Place {
-      local: place.local,
-      projection: tcx.intern_place_elems(&place.projection[..deref_index]),
-    },
-    &place.projection[deref_index + 1..],
-  ))
+pub fn is_direct(place: Place<'tcx>, body: &Body<'tcx>) -> bool {
+  !place.is_indirect() || is_arg(place, body)
+}
+
+pub fn pointers_in_place(place: Place<'tcx>, tcx: TyCtxt<'tcx>) -> SmallVec<[Place<'tcx>; 2]> {
+  place
+    .iter_projections()
+    .filter_map(|(place_ref, elem)| match elem {
+      ProjectionElem::Deref => Some(mk_place(place_ref.local, place_ref.projection, tcx)),
+      _ => None,
+    })
+    .collect()
 }
