@@ -1,6 +1,6 @@
 use crate::{
   core::{
-    analysis::{FlowistryAnalysis, FlowistryOutput, FlowistryResult},
+    analysis::{self, FlowistryAnalysis, FlowistryOutput, FlowistryResult},
     config::Range,
     indexed::IndexedDomain,
     utils,
@@ -75,14 +75,12 @@ impl FlowistryAnalysis for EffectsHarness {
   }
 
   fn analyze_function(&mut self, tcx: TyCtxt, body_id: BodyId) -> Result<Self::Output> {
-    let body_with_facts = utils::get_body_with_borrowck_facts(tcx, body_id);
+    let def_id = tcx.hir().body_owner_def_id(body_id);
+    let body_with_facts = analysis::get_body_with_borrowck_facts(tcx, def_id);
     let body = &body_with_facts.body;
     debug!("{}", utils::mir_to_string(tcx, body)?);
 
-    let flow_results = flow::compute_flow(tcx, body_id, &body_with_facts);
-    if std::env::var("DUMP_MIR").is_ok() {
-      utils::dump_results("target/effects.png", body, &flow_results)?;
-    }
+    let flow_results = &flow::compute_flow(tcx, body_id, body_with_facts);
 
     let mut find_effects = visitor::FindEffects::new(&flow_results.analysis);
     flow_results.visit_reachable_with(body, &mut find_effects);
@@ -102,12 +100,12 @@ impl FlowistryAnalysis for EffectsHarness {
       .unzip();
 
     let deps =
-      flow::compute_dependency_ranges(&flow_results, targets, Direction::Backward, &spanner);
+      flow::compute_dependency_ranges(flow_results, targets, Direction::Backward, &spanner);
 
     let source_map = tcx.sess.source_map();
     let mut ranged_effects = effects
       .into_iter()
-      .zip(deps.into_iter())
+      .zip(deps)
       .filter_map(|((kind, loc), slice)| {
         let spans = utils::location_to_spans(loc, body, &spanner, source_map);
         let range = spans
@@ -128,7 +126,7 @@ impl FlowistryAnalysis for EffectsHarness {
         .iter()
         .enumerate()
         .filter(|(j, _)| *j != i)
-        .map(|(_, (_, effect))| effect.slice.clone().into_iter())
+        .map(|(_, (_, effect))| effect.slice.clone())
         .flatten()
         .map(|range| (range.start..range.end, ()))
         .collect::<IntervalTree<_, _>>();
@@ -146,7 +144,7 @@ impl FlowistryAnalysis for EffectsHarness {
     }
 
     let body_span = Range::from_span(tcx.hir().body(body_id).value.span, source_map)?;
-    let fn_name = tcx.def_path_str(tcx.hir().body_owner_def_id(body_id).to_def_id());
+    let fn_name = tcx.def_path_str(def_id.to_def_id());
     let mut output = EffectsOutput {
       body_span,
       fn_name,
