@@ -19,14 +19,17 @@ extern crate rustc_graphviz;
 extern crate rustc_hir;
 extern crate rustc_hir_pretty;
 extern crate rustc_index;
+extern crate rustc_infer;
 extern crate rustc_interface;
 extern crate rustc_macros;
 extern crate rustc_middle;
 extern crate rustc_mir_dataflow;
 extern crate rustc_mir_transform;
 extern crate rustc_serialize;
+extern crate rustc_session;
 extern crate rustc_span;
 extern crate rustc_target;
+extern crate rustc_trait_selection;
 extern crate smallvec;
 
 mod core;
@@ -35,9 +38,10 @@ mod flow;
 mod slicing;
 
 pub use crate::core::{
-  analysis::{FlowistryError, FlowistryResult},
-  config::{Config, Range},
+  analysis::{get_body_with_borrowck_facts, override_queries, FlowistryError, FlowistryResult},
+  config::{self, Range},
+  extensions, utils,
 };
 pub use effects::{effects, FunctionIdentifier};
-pub use flow::Direction;
+pub use flow::{compute_dependencies, compute_flow, Direction};
 pub use slicing::{slice, SliceOutput};
