@@ -1,6 +1,6 @@
 use crate::{
   core::{
-    analysis::{FlowistryAnalysis, FlowistryOutput, FlowistryResult},
+    analysis::{self, FlowistryAnalysis, FlowistryOutput, FlowistryResult},
     config::Range,
     utils,
   },
@@ -39,13 +39,13 @@ impl SliceOutput {
 
 impl FlowistryOutput for SliceOutput {
   fn merge(&mut self, other: SliceOutput) {
-    self.ranges.extend(other.ranges.into_iter());
+    self.ranges.extend(other.ranges);
     self.num_instructions = other.num_instructions;
     self.num_relevant_instructions = other.num_relevant_instructions;
     self.mutated_inputs = other.mutated_inputs;
     self.relevant_inputs = other.relevant_inputs;
     self.body_span = other.body_span;
-    self.sliced_spans.extend(other.sliced_spans.into_iter());
+    self.sliced_spans.extend(other.sliced_spans);
   }
 }
 
@@ -57,14 +57,11 @@ impl FlowistryAnalysis for ForwardSliceAnalysis {
   }
 
   fn analyze_function(&mut self, tcx: TyCtxt, body_id: BodyId) -> Result<Self::Output> {
-    let body_with_facts = utils::get_body_with_borrowck_facts(tcx, body_id);
+    let def_id = tcx.hir().body_owner_def_id(body_id);
+    let body_with_facts = analysis::get_body_with_borrowck_facts(tcx, def_id);
     let body = &body_with_facts.body;
-    debug!("{}", utils::mir_to_string(tcx, body)?);
 
-    let results = flow::compute_flow(tcx, body_id, &body_with_facts);
-    if std::env::var("DUMP_MIR").is_ok() {
-      utils::dump_results("target/flow.png", body, &results)?;
-    }
+    let results = &flow::compute_flow(tcx, body_id, body_with_facts);
 
     let source_map = tcx.sess.source_map();
     let (sliced_places, sliced_spans) =
@@ -72,14 +69,15 @@ impl FlowistryAnalysis for ForwardSliceAnalysis {
     debug!("sliced_places {:?}", sliced_places);
 
     let spanner = utils::HirSpanner::new(tcx, body_id);
-    let deps = flow::compute_dependency_ranges(&results, sliced_places, self.direction, &spanner);
+    let deps = flow::compute_dependency_ranges(results, sliced_places, self.direction, &spanner);
 
     let body_span = Range::from_span(tcx.hir().body(body_id).value.span, source_map)?;
     let sliced_spans = sliced_spans
       .into_iter()
       .map(|span| Range::from_span(span, source_map))
       .collect::<Result<Vec<_>>>()?;
-    let ranges = deps.into_iter().map(|v| v.into_iter()).flatten().collect();
+    let ranges = deps.into_iter().flatten().collect::<Vec<_>>();
+    debug!("found {} ranges in slice", ranges.len());
 
     Ok(SliceOutput {
       body_span,
