@@ -83,7 +83,7 @@ fn color_ranges(prog: &str, all_ranges: Vec<(&str, &HashSet<Range>)>) -> String
               .any(|other| range != other && other.start <= range.end && range.end < other.end)
           });
           let end_marker = if contained { "]" } else { "\x1B[0m]" };
-          vec![("[\x1B[31m", range.start), (end_marker, range.end)].into_iter()
+          [("[\x1B[31m", range.start), (end_marker, range.end)]
         })
         .flatten()
     })
@@ -114,8 +114,8 @@ fn compare_ranges(expected: HashSet<Range>, actual: HashSet<Range>, prog: &str)
     }
   };
 
-  check(missing, "Actual was missing");
-  check(extra, "Actual contained unexpected");
+  check(missing, "Analysis did NOT have EXPECTED");
+  check(extra, "Actual DID have UNEXPECTED");
 }
 
 pub fn flow<O: Debug>(
@@ -157,7 +157,7 @@ pub fn slice(prog: &str, direction: Direction) {
     f.as_file_mut().write(prog_clean.as_bytes())?;
 
     let args = format!(
-      "--edition=2018 --crate-name tmp {} -A warnings --sysroot {}",
+      "rustc --crate-name tmp --edition=2018 {} -A warnings --sysroot {}",
       f.path().display(),
       *SYSROOT
     );
@@ -176,6 +176,9 @@ pub fn slice(prog: &str, direction: Direction) {
 }
 
 pub fn backward_slice(prog: &str) {
+  // use fluid_let::fluid_set;
+  // use flowistry::extensions::{ContextMode, EvalMode, EVAL_MODE};
+  // fluid_set!(EVAL_MODE, EvalMode { context_mode: ContextMode::Recurse, ..Default::default() });
   slice(prog, Direction::Backward);
 }
 
